@@ -1,7 +1,6 @@
 # Copyright (c) 2019 Ansible Project
 # GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 
 class ModuleDocFragment(object):
@@ -9,13 +8,13 @@ class ModuleDocFragment(object):
     # Windows shell documentation fragment
     # FIXME: set_module_language don't belong here but must be set so they don't fail when someone
     #  get_option('set_module_language') on this plugin
-    DOCUMENTATION = """
+    DOCUMENTATION = r"""
 options:
   async_dir:
     description:
     - Directory in which ansible will keep async job information.
-    - Before Ansible 2.8, this was set to C(remote_tmp + "\\.ansible_async").
-    default: '%USERPROFILE%\\.ansible_async'
+    - Before Ansible 2.8, this was set to C(remote_tmp + "\.ansible_async").
+    default: '%USERPROFILE%\.ansible_async'
     ini:
     - section: powershell
       key: async_dir
@@ -35,14 +34,8 @@ options:
     description:
     - Controls if we set the locale for modules when executing on the
       target.
-    - Windows only supports C(no) as an option.
+    - Windows only supports V(no) as an option.
     type: bool
     default: 'no'
     choices: ['no', False]
-  environment:
-    description:
-    - List of dictionaries of environment variables and their values to use when
-      executing commands.
-    type: list
-    default: [{}]
 """
