@@ -73,40 +73,32 @@ _value:
   elements: raw
 """
 
-from ansible.errors import AnsibleError, AnsibleUndefinedVariable
-from ansible.module_utils.six import string_types
+from ansible.errors import AnsibleTypeError
 from ansible.plugins.lookup import LookupBase
+from ansible.module_utils.datatag import native_type_name
+from ansible._internal._templating import _jinja_bits
 
 
 class LookupModule(LookupBase):
-
-    def run(self, terms, variables=None, **kwargs):
-        if variables is not None:
-            self._templar.available_variables = variables
-        myvars = getattr(self._templar, '_available_variables', {})
-
+    def run(self, terms, variables, **kwargs):
         self.set_options(var_options=variables, direct=kwargs)
+
         default = self.get_option('default')
 
         ret = []
+
         for term in terms:
-            if not isinstance(term, string_types):
-                raise AnsibleError('Invalid setting identifier, "%s" is not a string, its a %s' % (term, type(term)))
+            if not isinstance(term, str):
+                raise AnsibleTypeError(f'Variable name must be {native_type_name(str)!r} not {native_type_name(term)!r}.', obj=term)
 
             try:
-                try:
-                    value = myvars[term]
-                except KeyError:
-                    try:
-                        value = myvars['hostvars'][myvars['inventory_hostname']][term]
-                    except KeyError:
-                        raise AnsibleUndefinedVariable('No variable found with this name: %s' % term)
-
-                ret.append(self._templar.template(value, fail_on_undefined=True))
-            except AnsibleUndefinedVariable:
-                if default is not None:
-                    ret.append(default)
+                value = variables[term]
+            except KeyError:
+                if default is None:
+                    value = _jinja_bits._undef(f'No variable named {term!r} was found.')
                 else:
-                    raise
+                    value = default
 
-        return ret
+            ret.append(value)
+
+        return self._templar._engine.template(ret)
